// Convert array to sol (Original converted from XML)
// Ported from http://osflash.org/s2x (Public Domain, No Licencing issues.)
// by Trevor Hobson
// Added Pointer data type - TH
var objectionSolWrite = {
	NUMBER: '\x00',
	BOOLEAN: '\x01',
	STRING: '\x02',
	OBJOBJ: '\x03',
	NULL: '\x05',
	UNDEF: '\x06',
	POINTER: '\x07',
	OBJARR: '\x08',
	//RAWARR: '\x0A',
	OBJDATE: '\x0B',
	OBJM: '\x0D',
	OBJXML: '\x0F',
	OBJCC: '\x10',
	contentArray: [],
	countElement: 0,
	count: 0,

	writer: function(solFile, solArray)
	{
		objectionSolWrite.countElement = 1;
		objectionSolWrite.contentArray = new Array();
		objectionSolWrite.contentArray = solArray;
		try
		{
			if (solArray[0].type != 'name')
			{
				return null;
			}
			return objectionSolWrite.writeSol(solFile);
		} catch(er)
		{
			return null;
		}
	},
		
	writeSol: function(solFile)
	{
		// Create storage stream (temporary stream to hold content)
		var sstream = Components.classes["@mozilla.org/storagestream;1"].createInstance(Components.interfaces.nsIStorageStream);
		sstream.init(128, -1, null);

		var bstreamSO = Components.classes["@mozilla.org/binaryoutputstream;1"].createInstance(Components.interfaces.nsIBinaryOutputStream);
		bstreamSO.setOutputStream(sstream);

		bstreamSO.writeBytes('TCSO', 4); // sFileType
		bstreamSO.writeBytes('\x00\x04\x00\x00\x00\x00', 6); // sth1
		var solname = objectionSolWrite.contentArray[0].name;
		var nLenSoln = solname.length;
		bstreamSO.write16(nLenSoln);
		bstreamSO.writeBytes(solname, nLenSoln);
		bstreamSO.writeBytes('\x00\x00\x00\x00', 4); // sth2

		var isError = false;
		while (objectionSolWrite.countElement < objectionSolWrite.contentArray.length && isError === false)
		{
			isError = objectionSolWrite.makeStream(bstreamSO);
			if (!isError)
			{
				bstreamSO.writeBytes('\x00', 1);
			}
			objectionSolWrite.countElement++;
		}
		if (isError === true)
		{
			sstream.close;
			return null;
		}

		// Create input stream to read storage stream		
		var istreamS = sstream.newInputStream(0);
		var bstreamSI = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
		bstreamSI.setInputStream(istreamS);
		
		nLenData = bstreamSI.available();
		var sData = bstreamSI.readByteArray(nLenData);
		
		// Close the storage stream as we don't need it any more
		sstream.close;
		
		// Create the file if it does not exist
		if (!solFile.exists())
		{
			solFile.create(0,0666);
		}
		// Open the file as an output stream
		var ostream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		ostream.init(solFile,-1, -1, false);
		var bstreamO = Components.classes["@mozilla.org/binaryoutputstream;1"].createInstance(Components.interfaces.nsIBinaryOutputStream);
		bstreamO.setOutputStream(ostream);
		
		bstreamO.writeBytes('\x00\xBF', 2); // sHeader
		bstreamO.write32(nLenData);
		bstreamO.writeByteArray(sData, nLenData);
				
		ostream.close();
		return true;
	},
	
	makeStream: function(bstreamSO)
	{
		var sVname = objectionSolWrite.contentArray[objectionSolWrite.countElement].name;
		var nLenVname = sVname.length;
		bstreamSO.write16(nLenVname);
		bstreamSO.writeBytes(sVname, nLenVname);
		
		var isError = false;
		var aType = objectionSolWrite.contentArray[objectionSolWrite.countElement].type;
		if (aType == 'number')
		{
			isError = objectionSolWrite.a2num(bstreamSO);
		}
		else if (aType == 'boolean')
		{
			isError = objectionSolWrite.a2bol(bstreamSO);
		}
		else if (aType == 'string')
		{
			isError = objectionSolWrite.a2str(bstreamSO);
		}
		else if (aType == 'object')
		{
			isError = objectionSolWrite.a2obj(bstreamSO);
		}
		else if (aType == 'array')
		{
			isError = objectionSolWrite.a2arr(bstreamSO);
		}
		else if (aType == 'date')
		{
			isError = objectionSolWrite.a2dat(bstreamSO);
		}
		else if (aType == 'xml')
		{
			isError = objectionSolWrite.a2xml(bstreamSO);
		}
		else if (aType == 'c_object')
		{
			isError = objectionSolWrite.a2occ(bstreamSO);
		}
		else if (aType == 'm_object')
		{
			isError = objectionSolWrite.a2ojm(bstreamSO);
		}
		else if (aType == 'pointer')
		{
			isError = objectionSolWrite.a2ptr(bstreamSO);
		}
		else if (aType == 'null')
		{
			isError = objectionSolWrite.a2nul(bstreamSO);
		}
		else if (aType == 'undefined')
		{
			isError = objectionSolWrite.a2und(bstreamSO);
		}
		else
		{
			isError = true;
		}
		return isError;
	},
	
	a2num: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.NUMBER, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'Infinity')
		{
			bstreamSO.writeBytes('\x7F\xF0\x00\x00\x00\x00\x00\x00', 8);
		}
		else if (sValue == '-Infinity')
		{
			bstreamSO.writeBytes('\xFF\xF0\x00\x00\x00\x00\x00\x00', 8);
		}
		else if (sValue == 'NaN')
		{
			bstreamSO.writeBytes('\x7F\xF8\x00\x00\x00\x00\x00\x00', 8);
		}
		else if (sValue == 'error')
		{
			isError = true;
		}
		else
		{
			bstreamSO.writeDouble(sValue);
		}
		return isError;
	},
	
	a2bol: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.BOOLEAN, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'true')
		{
			bstreamSO.writeBytes('\x01', 1);
		}
		else if (sValue == 'false')
		{
			bstreamSO.writeBytes('\x00', 1);
		}
		else
		{
			isError = true;
		}
		return isError;
	},

	a2str: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.STRING, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'error')
		{
			isError = true;
		}
		else
		{
			var nLenStr = sValue.length;
			bstreamSO.write16(nLenStr);
			bstreamSO.writeBytes(sValue, nLenStr);
		}
		return isError;
	},
	
	a2obj: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.OBJOBJ, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'error')
		{
			isError = true;
		}
		else
		{
			var level = objectionSolWrite.contentArray[objectionSolWrite.countElement].level;
			objectionSolWrite.countElement++;
			while (objectionSolWrite.countElement < objectionSolWrite.contentArray.length && objectionSolWrite.contentArray[objectionSolWrite.countElement].level > level && isError === false)
			{
				isError = objectionSolWrite.makeStream(bstreamSO);
				objectionSolWrite.countElement++;
			}
			objectionSolWrite.countElement--;
			if (!isError)
			{
				bstreamSO.writeBytes('\x00\x00\x09', 3);
			}
		}
		return isError;
	},
	
	a2arr: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.OBJARR, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'error')
		{
			isError = true;
		}
		else
		{
			var level = objectionSolWrite.contentArray[objectionSolWrite.countElement].level;
			var countArray = objectionSolWrite.countElement + 1;
			var nArrLength = 0;
			while (countArray < objectionSolWrite.contentArray.length && objectionSolWrite.contentArray[countArray].level > level)
			{
				if (objectionSolWrite.contentArray[countArray].level == level + 1)
				{
					nArrLength++;
				}
				countArray++;
			}
			bstreamSO.write32(nArrLength);
			objectionSolWrite.countElement++;
			while (objectionSolWrite.countElement < objectionSolWrite.contentArray.length && objectionSolWrite.contentArray[objectionSolWrite.countElement].level > level && isError === false)
			{
				isError = objectionSolWrite.makeStream(bstreamSO);
				objectionSolWrite.countElement++;
			}
			objectionSolWrite.countElement--;
			if (!isError)
			{
				bstreamSO.writeBytes('\x00\x00\x09', 3);
			}
		}
		return isError;
	},
	
	a2dat: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.OBJDATE, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'error')
		{
			isError = true;
		}
		else
		{
			var sDate = sValue.match(/.*(?=\sUTC)/i);
			var nDate = Date.parse(sDate);
			var sOffset = sValue.match(/UTC.*/i).replace(/UTC/i,'');
			var nMinOffset = parseInt(sOffset, 10) * -60;
			bstreamSO.writeDouble(nDate);
			bstreamSO.write16(nMinOffset);
		}
		return isError;
	},

	a2xml: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.OBJXML, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'error')
		{
			isError = true;
		}
		else
		{
			var nLenXml = sValue.length;
			bstreamSO.write32(nLenXml);
			bstreamSO.writeBytes(sValue, nLenXml);
		}
	},
	
	a2occ: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.OBJCC, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'error')
		{
			isError = true;
		}
		else
		{
			var level = objectionSolWrite.contentArray[objectionSolWrite.countElement].level;
			objectionSolWrite.countElement++;
			while (objectionSolWrite.countElement < objectionSolWrite.contentArray.length && objectionSolWrite.contentArray[objectionSolWrite.countElement].level > level && isError === false)
			{
				isError = objectionSolWrite.makeStream(bstreamSO);
				objectionSolWrite.countElement++;
			}
			objectionSolWrite.countElement--;
			if (!isError)
			{
				bstreamSO.writeBytes('\x00\x00\x09', 3);
			}
		}
		return isError;
	},
	
	a2ojm: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.OBJM, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'error')
		{
			isError = true;
		}
		return isError;
	},

	a2ptr: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.POINTER, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'error')
		{
			isError = true;
		}
		else
		{
			bstreamSO.write16(sValue);
		}
		return isError;
	},
	
	a2nul: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.NULL, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'error')
		{
			isError = true;
		}
		return isError;
	},
	
	a2und: function(bstreamSO)
	{
		bstreamSO.writeBytes(objectionSolWrite.UNDEF, 1);
		var isError = false;
		var sValue = objectionSolWrite.contentArray[objectionSolWrite.countElement].value;
		if (sValue == 'error')
		{
			isError = true;
		}
		return isError;
	}
};